

import java.util.List;
import java.util.Iterator;

public class ChordNamer { 



    public ChordNamer ( Scale scale ) { 
	this.scale = scale;
    }

    protected final Scale scale;
    public Scale getScale() { return scale; }
	
    public String[] nameChord ( final List tones ) { 
	final Scale s = getScale();
	final int n = s.getNumTones();
	Tone t = s.getRoot();

	final String[] out = new String[n];

	for ( int i = 0; i < n; ++i ) { 
	    out[i] = tryNameChord( s, t, tones );
	    t = s.nextTone( t );
	}
	return out;
    }

    protected String tryNameChord ( final Scale s,
				    final Tone root,
				    final List tones ) { 
	final boolean b[] = new boolean[ s.getNumTones() ];
	for ( int i = 0; i < b.length; ++i ) 
	    b[i] = false;
	
	for ( final Iterator i = tones.iterator(); i.hasNext(); ) { 
	    final Tone t = (Tone)i.next();
	    if ( t != null ) { 
		final int ival = s.getNormalInterval( root, t );
		if ( ival < 0 ) System.err.println("BAH - neg interval");
		b[ival] = true;
	    }
	}

	final boolean has_root = b[0];
	
	final boolean minor_color = b[3];
	final boolean major_color = b[4];
	final boolean no_3rd = !(minor_color || major_color);
	
	final boolean has_5th = b[7];
	final boolean has_flat5th = b[6];
	final boolean has_aug5th = b[8];

	final boolean diminished = major_color && has_flat5th;
	final boolean augmented = minor_color && has_aug5th;

	final boolean add2 = b[2];
	final boolean add4 = b[5];
	
	final boolean sus2 = !major_color && add2;
	final boolean sus4 = !major_color && add4;
	
	final boolean has_min7th = b[10];
	final boolean has_maj7th = b[11];
	final boolean has_6th = b[9];

	final boolean seventh = major_color && has_min7th;
	final boolean maj_seventh = major_color && has_maj7th;
	final boolean min_seventh = minor_color && has_min7th;

	final String tonename = s.nameTone( root );

	String name = tonename;

	if ( major_color || minor_color ) {
	    if ( seventh ) name += "7";
	    else if ( maj_seventh ) name += "M7";
	    else if ( min_seventh ) name += "m7";
	    else if ( major_color ) name += "maj";
	    else if ( minor_color ) name += "min";


	    if ( add2 ) name += " add2";
	    else if ( add4 ) name += " add4";

	    if ( has_flat5th ) name += " flat5";
	    if ( has_aug5th ) name += " aug5";
	} else {
	    if ( diminished || augmented ) { 
		if ( diminished && augmented ) name += "(dim/aug)";
		else if ( diminished ) name += "dim";
		else if ( augmented ) name += "aug";

		if ( add2 ) name += " add2";
		else if ( add4 ) name += " add4";
	    } else { 
		if ( sus4 ) name += " sus4";
		if ( sus2 ) name += " sus2";

		if ( has_flat5th ) name += " flat5";
		if ( has_aug5th ) name += " aug5";
	    }

	    if ( has_min7th ) name += " m7";
	    if ( has_maj7th ) name += " M7";
	}

	if ( has_6th ) name += " add6";
	if ( !has_5th ) name += " no5";
	if ( !has_root ) name += " no1";

	if ( !has_root && !has_5th &&
	     (!major_color && !minor_color) ) 
	    name = "";

	return name;
	
    }


}
